<?php

namespace FiloBlu\StockaFix\Cron;

use Exception;
use FiloBlu\StockaFix\Model\Report\ProcessorInterfaceFactory;
use FiloBlu\StockaFix\Model\ReportInterface;
use FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory;
use Psr\Log\LoggerInterface;

class Export
{
    protected $logger;
    /**
     * @var CollectionFactory
     */
    private $reportCollectionFactory;
    /**
     * @var ProcessorInterfaceFactory
     */
    private $processorFactory;

    /**
     * Constructor
     *
     * @param LoggerInterface $logger
     * @param CollectionFactory $reportCollectionFactory
     * @param ProcessorInterfaceFactory $processorFactory
     */
    public function __construct(
        LoggerInterface $logger,
        CollectionFactory $reportCollectionFactory,
        ProcessorInterfaceFactory $processorFactory
    ) {
        $this->logger = $logger;
        $this->reportCollectionFactory = $reportCollectionFactory;
        $this->processorFactory = $processorFactory;
    }

    /**
     * Execute the cron
     *
     * @return void
     */
    public function execute()
    {
        $collection = $this->reportCollectionFactory->create();
        $this->connection = $collection->getResource()->getConnection();

        $reports = $collection
            ->addFieldToFilter('active', ['eq' => 1])
            ->getItems();

        /** @var ProcessorInterface $processor */
        $processor = $this->processorFactory->create();

        /** @var ReportInterface $report */
        foreach ($reports as $report) {
            if (!$processor->canProcess($report)) {
                continue;
            }

            try {
                $processor->process($report);
            } catch (Exception $exception) {
                $this->logger->error($exception);
            }
        }
    }
}
